# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2015-2016 Tiago Baptista
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# -----------------------------------------------------------------------------

"""
This module provides several ready-made simulator classes. Used mainly for the
examples given in the Complex Systems course. In this current version, these
should *not* be considered stable in terms of API.

"""

from __future__ import division
from simcx import Simulator

__docformat__ = 'restructuredtext'
__author__ = 'Tiago Baptista'


class FunctionIterator(Simulator):
    def __init__(self, func, initial_states):
        super(FunctionIterator, self).__init__()
        self._state = [state for state in initial_states]
        self._n_states = len(self._state)
        self.func = func
        self.time = 0
        self.x = [0]
        self.y = [[state] for state in initial_states]

    def step(self, delta=0):
        self.time += 1
        for i in range(self._n_states):
            self._state[i] = self.func(self._state[i])
            self.y[i].append(self._state[i])
        self.x.append(self.time)


class FunctionIterator2D(Simulator):
    def __init__(self, func, initial_state):
        super(FunctionIterator2D, self).__init__()

        self._func = func
        self._state = initial_state
        self.time = 0
        self.x = [0]
        self.y = [[initial_state[0]], [initial_state[1]]]

    def step(self, delta=0):
        self.time += 1
        self._state = self._func(*self._state)
        self.x.append(self.time)
        self.y[0].append(self._state[0])
        self.y[1].append(self._state[1])

